@@ -1,18 +1,20 @@
 /*
  * Name - checkvm.h
  *
- * Version:	1.3
+ * Version:	1.4
  *
- * ccsid:	@(#)checkvm.h	1.3 - 06/25/93 12:54:17
+ * ccsid:	@(#)checkvm.h	1.4 - 08/16/95 08:16:34
  * from: 	ccs/s.checkvm.h
- * date: 	06/28/93 09:14:48
+ * date: 	11/07/95 11:31:39
  *
  * description:  a routine to check resource availability before attempting
  *               to display a stand-alone image
  */
 
 
-char *CheckVM[] = {
+EXT char *CheckVM[]
+#if defined(XGRABSC_C)
+  = {
   "/checkvm { %% bytesNeeded checkvm -",
   "  /needed exch store",
   "  /used  0 store",
@@ -35,6 +37,8 @@ char *CheckVM[] = {
   "  } if",
   "} def",
   0
-  };
+  }
+#endif
+  ;
 
 
