@@ -1,14 +1,15 @@
+#define XGRABSC_C
 /*========================================================================
  *
  * Name - xgrabsc.c
  *
- * ccs version:	1.31
+ * ccs version:	1.40
  *
- * ccsid:	@(#)xgrabsc.c	1.31 - 06/28/93 09:13:50
+ * ccsid:	@(#)xgrabsc.c	1.40 - 08/16/95 08:16:44
  * from: 	ccs/s.xgrabsc.c
- * date: 	06/28/93 09:14:49
+ * date: 	11/07/95 11:31:40
  *
- * Copyright (c) 1990-93 Bruce Schuchardt.
+ * Copyright (c) 1990-95 Bruce Schuchardt.
  * Read the file cpyright.h for full copyright information.
  *
  * XmuClientWindow and TryChildren are Copyright 1989 by the Massachusetts
@@ -32,148 +33,7 @@
  *========================================================================
  */
 
-#if defined(__hpux)
-/* the folks at HP have decided to make things more difficult
- */
-#define XLIB_ILLEGAL_ACCESS
-#endif
-
-
-#include "cpyright.h"
-#include "patchlevel.h"
-#include "config.h"
-#include "checkvm.h"
-#include "cmdopts.h"
-
-#include <stdio.h>
-
-#include <X11/Xos.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/cursorfont.h>
-#include <X11/Xatom.h>
-
-#ifdef VMS
-#include "XWDFile.h"
-#else
-#include <X11/XWDFile.h>
-#endif /* VMS */
-
-#ifndef CARD32
-#include <X11/Xmd.h>
-#endif
-
-
-#ifndef NO_VIRTUAL_WINDOW
-#include "virtual.h"
-#endif
-
-
-#define MAX_CELLS  256
-#define TRUE  1
-#define FALSE 0
-
-
-typedef enum {
-  NO_DITHER=0,
-  FS_DITHER,
-  MATRIX_DITHER,
-  MATRIX_HALFTONE,
-  MAP_BW
-} ditherType;
-
-
-typedef unsigned char byte;
-typedef unsigned long dw;
-typedef unsigned int  word;
-
-
-
-typedef struct {
-  XImage *ximage;
-  word numcells;
-  word red[MAX_CELLS], green[MAX_CELLS], blue[MAX_CELLS];
-  byte used[MAX_CELLS];
-} imageInfo;
-
-
-typedef enum {
-  sourceRect,
-  sourceRoot,
-  sourceId,
-  sourceWd,
-  sourceKey,
-  sourceCoords
-} sourceType;
-
-typedef enum {
-  puzzleFormat,
-  psFormat,
-  simpleFormat,
-  xwdFormat,
-  pixmapFormat
-} formatType;
-
-
-
-
-#ifdef BCOPY
-#ifdef MEMCPY
-#undef MEMCPY
-#endif
-#define memcpy(x,y,c) bcopy(y,x,c)
-#define memset(x,c)   bzero(x,c) /* well, I only use it for zeroing out stuff */
-#endif
-
-#ifdef MEMCPY
-char *memcpy();
-char *memset();
-#endif
-
-
-#ifdef VMS
-#define rindex strrchr
-#endif
-
-static Display *hDisplay;
-static int      hScreen;
-static Window   hRoot, vRoot;
-static int      displayCells;
-static char    *programName;
-static char    imageName[64];
-
-static char    *version    = XGRABSC_VERSION;
-static int      patchLevel = XGRABSC_PATCHLEVEL;
-static int      verbose;
-
-static word nr[MAX_CELLS], ng[MAX_CELLS], nb[MAX_CELLS];
-
-static char   hexdigits[] = "0123456789abcdef";
-
-static double pageWidth   = PAPER_WIDTH;
-static double pageHeight  = PAPER_HEIGHT;
-static double horizMargin = HORIZ_MARGIN;
-static double vertMargin  = VERT_MARGIN;
-static double horizInset  = PAPER_WIDTH - HORIZ_MARGIN - HORIZ_MARGIN;
-static double vertInset   = PAPER_HEIGHT - VERT_MARGIN - VERT_MARGIN;
-
-static char        *outfileName;
-
-ditherType  DitherKind;
-int         Halftone;
-int         ForceBitmap;
-
-static int needColorImageProc;
-
-
-/* most functions have been moved to seperate source include files */
-
-#include "process.hc"         /* color processing     */
-#include "get.hc"             /* get images           */
-#include "mem.hc"             /* memcpy, memset       */
-#include "convert.hc"         /* color->bw conversion */
-#include "write.hc"           /* output functions     */
-
+#include "xgrabsc.h"
 
 
 
@@ -247,16 +107,52 @@ int ScreenNumberOfScreen (scr)
     register Screen *scr;
 {
     register Display *dpy = scr->display;
-    register Screen *dpyscr = dpy->screens;
     register int i;
 
-    for (i = 0; i < dpy->nscreens; i++, dpyscr++) {
-	if (scr == dpyscr) return i;
+    for (i = 0; i < ScreenCount(dpy); i++) {
+	if (scr == ScreenOfDisplay(dpy,i)) return i;
     }
     return -1;
 }
 
 
+Window windowNamed(startingWindow, name)
+  Window startingWindow;
+  char *name;
+{
+  int i;
+  unsigned int nchildren;
+  Window root, parent, *children, found;
+  char *windowName;
+  
+  if (!XQueryTree(hDisplay, startingWindow, &root, &parent, &children, &nchildren)) {
+    fprintf (stderr, "%s: unable to find top level windows\n", programName);
+    exit (3);
+  }
+  else {
+    found = (Window)NULL;
+    for (i=0; i<nchildren; i++) {
+      XFetchName(hDisplay, children[i], &windowName);
+      if (windowName) {
+	if (strcmp(name, windowName) == 0) {
+	  found = children[i];
+	  break;
+	}
+      }
+      else {
+	if (found = windowNamed(children[i], name))
+	  break;
+      }
+    }
+    if (children)
+      XFree((char *)children);
+  }
+
+  return found;
+}
+
+
+
 
 xgrabsc(argc, argv, env)
   int argc;
@@ -266,7 +162,7 @@ xgrabsc(argc, argv, env)
   char        *args[100], *arg;
   extern char *optarg;
   FILE        *outfile;
-  XRectangle  xrect;
+  XRectangle  xrect, xoffset;
   XWindowAttributes xwa;
   imageInfo   image;
   int         argn, argi, polarity;
@@ -274,7 +170,6 @@ xgrabsc(argc, argv, env)
   int         doAnd;
   int         doOr;
   int         doReverse;
-  int         depth;
   int         noBell;
   int         psColor;
   int         brighten;
@@ -285,7 +180,9 @@ xgrabsc(argc, argv, env)
   int         andBits;
   int         orBits;
   int         encapsulate;
+  int         subrectSource;
   Window      sourceWindow, childWindow, clientWindow, wmWindow, ignored;
+  Window      colorWin, dumyWin, imageWin;
   char       *ptr;
   char       *display;
   int         i, x, y;
@@ -296,10 +193,13 @@ xgrabsc(argc, argv, env)
   int         landscape;
   int         binary;
   int	      borders;
+  int         frame;
   int	      checkLimits;
   int	      preview;
   int         onlyEpsi;
   int         xwdxy;
+  int         absx, absy;
+  int         dwidth, dheight;
 
 
   outfile     = stdout;
@@ -320,10 +220,12 @@ xgrabsc(argc, argv, env)
   Halftone     = FALSE;
   landscape    = FALSE;
   binary       = FALSE;
+  subrectSource = FALSE;
   sleepSeconds = 0;
   postSelectSleepSeconds = 0;
   verbose      = FALSE;
   borders      = TRUE;
+  frame        = TRUE;
 #ifdef NO_PRINTER_MEMORY_CHECKS
   checkLimits  = FALSE;
 #else
@@ -337,7 +239,7 @@ xgrabsc(argc, argv, env)
   psColor      = FALSE;
   onlyEpsi     = FALSE;
   xwdxy	       = FALSE;
-
+  colorWin     = None;
 
   /* merge environment options and command line options */
   args[0] = programName;
@@ -421,8 +323,8 @@ xgrabsc(argc, argv, env)
       case CMD_BORDERS:
         borders = polarity;
 	break;
-      case CMD_NOBORDERS:
-        borders = !polarity;
+      case CMD_FRAME:
+        frame = polarity;
 	break;
 
 
@@ -431,9 +333,15 @@ xgrabsc(argc, argv, env)
         grabServer = FALSE;
 	break;
       case CMD_SOURCE_COORDS:
-      { short w,h,x,y;
+      { int x,y,parseResult;
+        unsigned int w,h;
 	source = sourceCoords;
-        sscanf(args[cmdargi], "%ix%i+%i+%i", &w, &h, &x, &y);
+	x = y = w = h = 0;
+	parseResult = XParseGeometry(args[cmdargi], &x, &y, &w, &h);
+	if ((parseResult & (WidthValue | HeightValue)) != (WidthValue | HeightValue)) {
+	  fprintf(stderr, "%s: coordinate rectangle underspecified\n", programName);
+	  exit(3);
+	}
 	xrect.width = w; 
 	xrect.height = h;
 	xrect.x = x;  
@@ -442,12 +350,6 @@ xgrabsc(argc, argv, env)
       }
 
       case CMD_SOURCE_ID:
-        source = sourceId;
-        sourceWindow = 0;
-        if (!sscanf(args[cmdargi], "%ix", &sourceWindow)) {
-          fprintf(stderr, "%s: invalid window id '%s'\n", programName, args[cmdargi]);
-	  exit(3);
-        }
         break;
       case CMD_SOURCE_RECT:
 	source = sourceRect;
@@ -459,7 +361,21 @@ xgrabsc(argc, argv, env)
         source = sourceWd;
         break;
 
+      case CMD_OFFSET_COORDS:
+      { int x,y,parseResult;
+        unsigned int w,h;
+	subrectSource = TRUE;
+	x = y = w = h = 0;
+	parseResult = XParseGeometry(args[cmdargi], &x, &y, &w, &h);
+	xoffset.width = w; 
+	xoffset.height = h;
+	xoffset.x = x;  
+	xoffset.y = y;
+	break;
+      }
 
+      case CMD_COLORWIN:
+        break;
 
       case CMD_AND:
         doAnd = polarity;
@@ -515,10 +431,6 @@ xgrabsc(argc, argv, env)
         Halftone = TRUE;
         ForceBitmap = FALSE;
         break;
-      case CMD_DITHER_NONE:
-        DitherKind = NO_DITHER;
-	Halftone = ForceBitmap = FALSE;
-	break;
 
 
 
@@ -560,6 +472,10 @@ xgrabsc(argc, argv, env)
         outputFormat = puzzleFormat;
         break;
 
+      case CMD_OUTPUT_PPM:
+        outputFormat = ppmFormat;
+        break;
+
       case CMD_LANDSCAPE:
         landscape = polarity;
 	outputFormat = psFormat;
@@ -604,12 +520,6 @@ xgrabsc(argc, argv, env)
   }
 
 
-  if (verbose) {
-    fprintf(stderr, "%s: xgrabsc version %s\n", programName, version);
-    fprintf(stderr, "%s:         patchlevel %d\n", programName, patchLevel);
-    fprintf(stderr, "%s:         %s\n\n", programName, Copyright);
-  }
-
   if (!display) display = (char *)getenv("DISPLAY");
   hDisplay = XOpenDisplay(display);
   if (!hDisplay) {
@@ -624,26 +534,93 @@ xgrabsc(argc, argv, env)
   vRoot    = hRoot;
 #endif
 
-  depth  = DefaultDepth(hDisplay, hScreen);
-  if (DisplayCells(hDisplay, hScreen) > MAX_CELLS) {
-    fprintf(stderr, "%s: color table is too big for this program\n",
-      programName);
-    XCloseDisplay(hDisplay);
-    exit(3);
+  /* process command line options that may require an X connection */
+  for (argi=1; argi<argn; argi++) {
+    arg = args[argi];
+
+    polarity = 1;
+    if (arg[0] == '-') {
+      arg++;
+      if (arg[0] == '-') {
+        arg++;
+        polarity = 0;
+      }
+      else if (arg[0] == 'n' && arg[1] == 'o') {
+        arg += 2;
+	polarity = 0;
+      }
+    }
+
+    for (cmdi=0; cmdi<numCmds; cmdi++) {
+      if (strcmp(arg, commands[cmdi].userstr) == 0)
+        break;
+    }
+
+    cmdargi = argi+1;
+    argi += commands[cmdi].numargs;
+
+    switch (commands[cmdi].command) {
+      case CMD_SOURCE_ID:
+        source = sourceId;
+        sourceWindow = 0;
+	if ( '0'<=(args[cmdargi][0])  && (args[cmdargi][0])<='9') {
+	  if (!sscanf(args[cmdargi], "%i", &sourceWindow)) {
+            fprintf(stderr, "%s: invalid window id '%s'\n", programName, args[cmdargi]);
+	    exit(3);
+	  }
+	}
+	else {
+	  sourceWindow = windowNamed(hRoot, args[cmdargi]);
+	  if (!sourceWindow) {
+	    fprintf(stderr, "%s: invalid window name '%s'\n", programName, args[cmdargi]);
+	    exit(3);
+	  }
+        }
+      case CMD_COLORWIN:
+        colorWin = (Window)NULL;
+	if ( '0'<=(args[cmdargi][0])  && (args[cmdargi][0])<='9') {
+	  if (!sscanf(args[cmdargi], "%i", &colorWin)) {
+            fprintf(stderr, "%s: invalid window id '%s'\n", programName, args[cmdargi]);
+	    exit(3);
+	  }
+	}
+	else {
+	  colorWin = windowNamed(hRoot, args[cmdargi]);
+	  if (!colorWin) {
+	    fprintf(stderr, "%s: invalid window name '%s'\n", programName, args[cmdargi]);
+	    exit(3);
+	  }
+        }
+	break;
+    }
   }
 
+  if (verbose) {
+    fprintf(stderr, "%s: xgrabsc version %s\n", programName, version);
+    fprintf(stderr, "%s:         patchlevel %d\n", programName, patchLevel);
+    fprintf(stderr, "%s:         %s\n\n", programName, Copyright);
+  }
+
+
   /* sleep if asked to do so */
   if (sleepSeconds)
     sleep(sleepSeconds);
 
+  if (verbose && grabServer) {
+    fprintf(stderr, "%s: disabling server grabs for verbose mode\n", programName);
+    grabServer = FALSE;
+  }
+
   /* grab the screen if asked to do so */
   if (grabServer)
     XGrabServer(hDisplay);
 
-  if (source != sourceId)
-    sourceWindow = hRoot;
+  if (source == sourceId)
+    childWindow = sourceWindow;
+  else
+    childWindow = (Window)NULL;
 
-  childWindow = (Window)NULL;
+  sourceWindow = hRoot;
 
   switch (source) {
     case sourceKey:
@@ -663,7 +640,6 @@ xgrabsc(argc, argv, env)
       /* already got the xrect in the command line processing section */
       break;
     case sourceId:
-      childWindow = sourceWindow;
       break;
     case sourceWd:
       /* grab the image from the root window so menus will show up on top
@@ -691,8 +667,9 @@ xgrabsc(argc, argv, env)
 
 
 
+  /* if frame removal has been requested, look for a "client" child window */
   clientWindow = (Window)NULL;
-  if (!borders  &&
+  if (!frame  &&
       (source == sourceKey || source == sourceWd || source == sourceId)  &&
       childWindow != hRoot) {
     /* look for a different client window */
@@ -705,7 +682,16 @@ xgrabsc(argc, argv, env)
     }
   }
 
-  if ((source == sourceKey && childWindow != hRoot) ||
+  /* Get the root's attributes, including colormap, depth and visual.
+   * This is overwritten if a child window is processed
+   */
+  if (!XGetWindowAttributes(hDisplay, hRoot, &xwa)) {
+    fprintf(stderr, "%s: unable to get root window attributes\n", programName);
+    XCloseDisplay(hDisplay);
+    exit(3);
+  }
+
+  if ((source == sourceKey) ||
       source == sourceId ||
       source == sourceWd) {
 
@@ -718,7 +704,7 @@ xgrabsc(argc, argv, env)
     else {
 
       /* take the child window (and optional client/wm window info) and
-      * determine the portion of the root window that should be grabbed */
+       * determine the portion of the root window that should be grabbed */
 
       if (!XGetWindowAttributes(hDisplay, childWindow, &xwa)) {
         fprintf(stderr, "%s: unable to get window coordinates\n", programName);
@@ -727,10 +713,10 @@ xgrabsc(argc, argv, env)
       }
 
       /* get the correct screen and root window for the selected window,
-        * and if it isn't the default, muck with the global state of the
-        * program a bit
-        */
-      i = ScreenNumberOfScreen(xwa.screen);
+       * and if it isn't the default, muck with the global state of the
+       * program a bit
+       */
+      i = XScreenNumberOfScreen(xwa.screen);
       if (i != hScreen) {
         hRoot   = xwa.root;
         hScreen = i;
@@ -739,57 +725,91 @@ xgrabsc(argc, argv, env)
 #else
         vRoot   = hRoot;
 #endif
-        depth   = DefaultDepth(hDisplay, hScreen);
       }
 
-      sourceWindow = vRoot;
-      if (!borders) {
-        xrect.x      = 0;
-        xrect.y      = 0;
-        xrect.width  = xwa.width;
-        xrect.height = xwa.height;
-      }
-      else {
-        xrect.x      = -xwa.border_width;
-        xrect.y      = -xwa.border_width;
-        xrect.width  = xwa.width + (2 * xwa.border_width);
-        xrect.height = xwa.height + (2 * xwa.border_width);
+      if (!XTranslateCoordinates (hDisplay, childWindow, vRoot, 0, 0,
+              &absx, &absy, &ignored)) {
+        fprintf(stderr,
+                "%s:  unable to translate window coordinates (%d,%d)\n",
+                        programName, absx, absy);
+        XCloseDisplay(hDisplay);
+        exit  (3);
       }
 
-      /* translate the rectangle coordinates to root-window coordinates */
-      if (!XTranslateCoordinates(hDisplay, childWindow, vRoot,
-            (int)(xrect.x), (int)(xrect.y), &x, &y, &ignored)) {
-	fprintf(stderr, "%s: unable to translate window coordinates\n", programName);
-	XCloseDisplay(hDisplay);
-	exit(3);
-      }
-      xrect.x = (short)x;   /* why did they make rectangle coords shorts? */
-      xrect.y = (short)y;
+      xrect.width = xwa.width;
+      xrect.height = xwa.height;
 
+      if (borders) {
+        absx         -= xwa.border_width;
+        absy         -= xwa.border_width;
+        xrect.width  += (2 * xwa.border_width);
+        xrect.height += (2 * xwa.border_width);
+      }
 
       if (hRoot == vRoot) {
-        /* do some clipping here, since it's cheap */
-        i = DisplayWidth(hDisplay, hScreen);
-        if (xrect.x + xrect.width > i) {
-          x = i - (int)(xrect.x);
-          if (x < 0) x = 0;
-          xrect.width = x;
+        /* do some clipping here */
+        if (absx < 0) {
+          xrect.width += absx;
+          absx = 0;
         }
-        i = DisplayHeight(hDisplay, hScreen);
-        if (xrect.y + xrect.height > i) {
-          x = i - (int)(xrect.y);
-          if (x < 0) x = 0;
-          xrect.height = x;
+        if (absy < 0) {
+          xrect.height += absy;
+          absy = 0;
         }
+        dwidth = DisplayWidth(hDisplay, hScreen);
+        dheight = DisplayHeight(hDisplay, hScreen);
+        if (absx + xrect.width > dwidth)
+          xrect.width = dwidth - absx;
+        if (absy + xrect.height > dheight)
+          xrect.height = dheight - absy;
       }
+
+      xrect.x = absx;
+      xrect.y = absy;
     }
   }
 
 
+  if (subrectSource) {
+    /* note that constraint checks here were left out intentionally so
+     * that you can grab an area outside of the selected window.  You
+     * should be able to grab the area to the left of the window, for
+     * instance */
+    xrect.x      += xoffset.x;
+    xrect.y      += xoffset.y;
+    xrect.width  = xoffset.width;
+    xrect.height = xoffset.height;
+    }
+
+  dwidth = DisplayWidth(hDisplay, hScreen);
+  dheight = DisplayHeight(hDisplay, hScreen);
+  if (xrect.x + xrect.width > dwidth)
+    xrect.width = dwidth - xrect.x;
+  if (xrect.y + xrect.height > dheight)
+    xrect.height = dheight - xrect.y;
+  
   if (verbose)
     fprintf(stderr, "%s: bounding box is [x=%d y=%d w=%d h=%d]\n", programName, xrect.x, xrect.y,
                     xrect.width, xrect.height);
 
+  /* get the depth, visual and colormap */
+  if (colorWin == None)
+    colorWin = childWindow? childWindow : sourceWindow;
+
+  if (!XGetWindowAttributes(hDisplay, colorWin, &xwa)) {
+    fprintf(stderr, "%s: unable to get window coordinates\n", programName);
+    XCloseDisplay(hDisplay);
+    exit(3);
+  }
+
+  image.depth    = xwa.depth;
+  image.visual   = xwa.visual;
+  image.colormap = xwa.colormap;
+  if (verbose)
+    fprintf(stderr, "%s: visual=%x depth=%d colormap=%x\n", programName,
+                    image.visual, image.depth, image.colormap);
+
+
   /* sleep if asked to do so */
   if (postSelectSleepSeconds) {
     if (grabServer) {
@@ -806,7 +826,7 @@ xgrabsc(argc, argv, env)
   if (!noBell)
     XBell(hDisplay, 50);
 
-  if (!getImage(&xrect, &image, sourceWindow, childWindow)) {
+  if (!getImage(&xrect, &image, sourceWindow)) {
     XCloseDisplay(hDisplay);
     exit(3);
   }
@@ -824,7 +844,7 @@ xgrabsc(argc, argv, env)
 
   /* do color image processing/conversions */
 
-  if (depth >= 2) {
+  if (image.depth >= 2) {
     if (brighten)
       brightenColors(&image, brightenFactor);
     if (doAnd)
@@ -836,11 +856,11 @@ xgrabsc(argc, argv, env)
   if (doReverse)
     reverseColors(&image);
 
-  if (depth >= 2) {
+  if (image.depth >= 2) {
     if (! (preview && psColor) ) {
       if (ForceBitmap) {
         pixmap2bitmap(&image);
-        depth = 1;
+        image.depth = 1;
       }
       else if (Halftone)
         pixmap2halftone(&image, DitherKind);
@@ -890,6 +910,9 @@ xgrabsc(argc, argv, env)
       else
         writeZPixmap(xpmFormat, &image, outfile);
       break;
+    case ppmFormat:
+      writePPM(&image, outfile);
+      break;
     case psFormat:
     default:
       if (onlyEpsi)
