@@ -2,19 +2,18 @@
  *
  * Name - xgrabxm.c
  *
- * Version:	1.5
+ * Version:	1.11
  *
- * ccsid:	@(#)xgrabxm.c	1.5 -06/28/93 09:13:55
+ * ccsid:	@(#)xgrabxm.c	1.11 -08/16/95 08:16:48
  * from: 	ccs/s.xgrabxm.c
- * date: 	06/28/93 09:14:49
+ * date: 	11/07/95 11:31:41
  *
- * Copyright (c) 1990-93 Bruce Schuchardt.
+ * Copyright (c) 1990-95 Bruce Schuchardt.
  * Read the file cpyright.h for full copyright information.
  *
- *
  * Description:
  *
- * xgrab.c - interactive front for xgrabsc
+ * xgrab.c - interactive front for xgrabsc (Motif version)
  *
  *========================================================================
  */
@@ -23,6 +22,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <ctype.h>
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
@@ -34,6 +34,27 @@
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
 
+
+/* for Motif 1.1 the XmuCvtStrToWidget converter must be added */
+#if XmVERSION == 1  &&  (XmREVISION == 1 || XmREVISION == 0)
+#define OLD_MOTIF
+
+#include <X11/IntrinsicP.h>
+#include <X11/StringDefs.h>
+#include <X11/ObjectP.h>
+
+extern void XmuCvtStringToWidget();
+
+static XtConvertArgRec parentCvtArgs[] = {
+   { 
+      XtWidgetBaseOffset, 
+      (XtPointer) XtOffsetOf(CoreRec, core.parent),
+      sizeof(CoreWidget)
+   }
+};
+#endif
+
+
 static char *fallback_resources[] = {
 #include "xgrabxm_ad.h"
   NULL
@@ -41,6 +62,8 @@ static char *fallback_resources[] = {
 
 
 static Display *hDisplay;
+static char    *version    = XGRABSC_VERSION;
+static int      patchLevel = XGRABSC_PATCHLEVEL;
 
 typedef enum {
   HITWIN = 1,
@@ -49,11 +72,12 @@ typedef enum {
   ROOT,
 
   XWD,
-  XWDXY,
   PS,
+  PPM,
+  BITMAP3,
   BITMAP,
   BITMAP2,
-  BITMAP3,
+  XWDXY,
   PUZZLE,
   SIMPLE,
 
@@ -107,7 +131,6 @@ void cnvValueChanged(widget, newValue, tag)
     cnvValue = newValue;
 }
 
-
 void selValueChanged(widget, newValue, tag)
   Widget widget;
   void *tag;
@@ -182,7 +205,11 @@ static char* stradw(ptr, s)   /* Add a string to another, with no ws */
 }
 
 
-static void doGrab() {
+static void doGrab(widget, newValue, tag)
+  Widget widget;
+  void *newValue;
+  void *tag;
+{
   char tmp[257];
   char cmdargs[200];
   char piper[200];
@@ -216,7 +243,10 @@ static void doGrab() {
   XtVaGetValues(borderBtn, XmNset, &state, NULL);
   if (!state)
     strcat(cmdargs, "-");
-  strcat(cmdargs, "-bdrs ");
+  strcat(cmdargs, "-borders ");
+  if (!state)
+    strcat(cmdargs, "-");
+  strcat(cmdargs, "-frame ");
 
   XtVaGetValues(revBtn, XmNset, &state, NULL);
   if (!state)
@@ -248,6 +278,9 @@ static void doGrab() {
     case BITMAP:
       strcat(cmdargs, "-bm ");
       break;
+    case PPM:
+      strcat(cmdargs, "-ppm ");
+      break;
     case BITMAP2:
       strcat(cmdargs, "-bm2 ");
       break;
@@ -403,7 +436,11 @@ static void doGrab() {
 
 
 
-static void doDismiss() {
+static void doDismiss(widget, newValue, tag)
+  Widget widget;
+  void *newValue;
+  void *tag;
+{
   if (hShell) {
     XtDestroyWidget(hShell);
     hShell = NULL;
@@ -418,280 +455,291 @@ static void createWindow() {
   Widget dialog, button, box1, box2, box3, box4, box5, box6;
 
   dialog = XtVaCreateManagedWidget("dialog", xmFormWidgetClass, hShell,
-           NULL);
-           XtVaCreateManagedWidget("title",  xmLabelWidgetClass, dialog,
-           NULL);
-
+           (XtPointer)NULL);
 
   /* selection options */
   /* input options */
-  box1 = XtVaCreateManagedWidget("box1", xmFormWidgetClass, dialog, NULL);
+  box1 = XtVaCreateManagedWidget("box1", xmFormWidgetClass, dialog, (XtPointer)NULL);
 
            XtVaCreateManagedWidget("inputLbl", xmLabelWidgetClass, box1,
-	   NULL);
+	   (XtPointer)NULL);
 
   dispBtn= XtVaCreateManagedWidget("host", xmToggleButtonWidgetClass, box1,
-	   NULL);
+	   (XtPointer)NULL);
 
   hostnm = XtVaCreateManagedWidget("hostText", xmTextWidgetClass, box1,
-           NULL);
+           (XtPointer)NULL);
 
   if (getenv("DISPLAY"))
-    XtVaSetValues(hostnm, XmNvalue, getenv("DISPLAY"), NULL);
+    XtVaSetValues(hostnm, XmNvalue, getenv("DISPLAY"), (XtPointer)NULL);
 
-  selBtns = XmCreateRadioBox(box1, "selectiontype", NULL, 0);
+  selBtns = XmCreateRadioBox(box1, "selectiontype", (XtPointer)NULL, 0);
            XtManageChild(selBtns);
 	   
   button = XtVaCreateManagedWidget("click", xmToggleButtonWidgetClass, selBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, selValueChanged,
-	   HITWIN);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)selValueChanged,
+	   (XtPointer)HITWIN);
 
   button = XtVaCreateManagedWidget("stretch", xmToggleButtonWidgetClass, selBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, selValueChanged,
-	   STRETCH);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)selValueChanged,
+	   (XtPointer)STRETCH);
 
   button = XtVaCreateManagedWidget("key", xmToggleButtonWidgetClass, selBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, selValueChanged,
-	   KEY);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)selValueChanged,
+	   (XtPointer)KEY);
 
   button = XtVaCreateManagedWidget("root", xmToggleButtonWidgetClass, selBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, selValueChanged,
-	   ROOT);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)selValueChanged,
+	   (XtPointer)ROOT);
 
            XtVaCreateManagedWidget("sleeplbl", xmLabelWidgetClass, box1,
-           NULL);
+           (XtPointer)NULL);
 
   sltext = XtVaCreateManagedWidget("sleeptime", xmTextWidgetClass, box1,
-           NULL);
+           (XtPointer)NULL);
 
            XtVaCreateManagedWidget("psleeplbl", xmLabelWidgetClass, box1,
-           NULL);
+           (XtPointer)NULL);
 
   psltext = XtVaCreateManagedWidget("psleeptime", xmTextWidgetClass, box1,
-           NULL);
+           (XtPointer)NULL);
 
-  box2 = XtVaCreateManagedWidget("box2", xmFormWidgetClass, dialog, NULL);
+  box2 = XtVaCreateManagedWidget("box2", xmFormWidgetClass, dialog, (XtPointer)NULL);
 
   /* output options */
            XtVaCreateManagedWidget("outputFormat", xmLabelWidgetClass, box2,
-           NULL);
+           (XtPointer)NULL);
 
   formatBtns = XmCreateRadioBox(box2, "formattype", NULL, 0);
 	   XtManageChild(formatBtns);
 	   
-  button = XtVaCreateManagedWidget("xwd", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   XWD);
-
-  button = XtVaCreateManagedWidget("xwdxy", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   XWDXY);
-
   button = XtVaCreateManagedWidget("ps", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   PS);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)PS);
 
-  button = XtVaCreateManagedWidget("xpm", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   BITMAP);
+  button = XtVaCreateManagedWidget("xwd", xmToggleButtonWidgetClass, formatBtns,
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)XWD);
 
-  button = XtVaCreateManagedWidget("xpm2", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   BITMAP2);
+  button = XtVaCreateManagedWidget("ppm", xmToggleButtonWidgetClass, formatBtns,
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)PPM);
 
   button = XtVaCreateManagedWidget("xpm3", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   BITMAP3);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)BITMAP3);
+
+  button = XtVaCreateManagedWidget("xpm", xmToggleButtonWidgetClass, formatBtns,
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)BITMAP);
+
+  button = XtVaCreateManagedWidget("xpm2", xmToggleButtonWidgetClass, formatBtns,
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)BITMAP2);
+
+  button = XtVaCreateManagedWidget("xwdxy", xmToggleButtonWidgetClass, formatBtns,
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)XWDXY);
 
   button = XtVaCreateManagedWidget("puzzle", xmToggleButtonWidgetClass, formatBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, fmtValueChanged,
-	   PUZZLE);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)fmtValueChanged,
+	   (XtPointer)PUZZLE);
 
 
 
   /* postscript options */
-  box3 = XtVaCreateManagedWidget("box3", xmFormWidgetClass, dialog, NULL);
+  box3 = XtVaCreateManagedWidget("box3", xmFormWidgetClass, dialog, (XtPointer)NULL);
 
            XtVaCreateManagedWidget("psOptions", xmLabelWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
   psComp = XtVaCreateManagedWidget("compress", xmToggleButtonWidgetClass, box3,
-	   NULL);
+	   (XtPointer)NULL);
 
   psColr = XtVaCreateManagedWidget("color", xmToggleButtonWidgetClass, box3,
-	   NULL);
+	   (XtPointer)NULL);
 
            XtVaCreateManagedWidget("pageWidth", xmLabelWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
   pgWidth = XtVaCreateManagedWidget("pageWidthText",xmTextWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
            XtVaCreateManagedWidget("pageHeight", xmLabelWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
   pgHeight = XtVaCreateManagedWidget("pageHeightText",xmTextWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
            XtVaCreateManagedWidget("horizMargin", xmLabelWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
   horizMargin = XtVaCreateManagedWidget("horizMarginText",xmTextWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
            XtVaCreateManagedWidget("vertMargin", xmLabelWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
   vertMargin = XtVaCreateManagedWidget("vertMarginText",xmTextWidgetClass, box3,
-           NULL);
+           (XtPointer)NULL);
 
 
   encBtns = XmCreateRadioBox(box3, "postscripttype", NULL, 0);
 	   XtManageChild(encBtns);
 	   
   button = XtVaCreateManagedWidget("portrait", xmToggleButtonWidgetClass, encBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, encValueChanged,
-	   PS_NORMAL);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)encValueChanged,
+	   (XtPointer)PS_NORMAL);
 
   button = XtVaCreateManagedWidget("landscape", xmToggleButtonWidgetClass, encBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, encValueChanged,
-	   PS_LANDSCAPE);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)encValueChanged,
+	   (XtPointer)PS_LANDSCAPE);
 
   button = XtVaCreateManagedWidget("epsi", xmToggleButtonWidgetClass, encBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, encValueChanged,
-	   PS_EPSI);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)encValueChanged,
+	   (XtPointer)PS_EPSI);
 
 /*------------
   button = XtVaCreateManagedWidget("encap", xmToggleButtonWidgetClass, encBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, encValueChanged,
-	   PS_EPS);
----------------*/
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)encValueChanged,
+	   (XtPointer)PS_EPS);
 
   button = XtVaCreateManagedWidget("onlyEpsi", xmToggleButtonWidgetClass, encBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, encValueChanged,
-	   PS_EPSIONLY);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)encValueChanged,
+	   (XtPointer)PS_EPSIONLY);
+---------------*/
 
   psBin = XtVaCreateManagedWidget("binary", xmToggleButtonWidgetClass, box3,
-	   NULL);
+	   (XtPointer)NULL);
 
   psLim = XtVaCreateManagedWidget("limit", xmToggleButtonWidgetClass, box3,
-	   NULL);
+	   (XtPointer)NULL);
 
 
   /* image processing options */
-  box4 = XtVaCreateManagedWidget("box4", xmFormWidgetClass, dialog, NULL);
+  box4 = XtVaCreateManagedWidget("box4", xmFormWidgetClass, dialog, (XtPointer)NULL);
 
            XtVaCreateManagedWidget("prOptions", xmLabelWidgetClass, box4,
-           NULL);
+           (XtPointer)NULL);
 
            XtVaCreateManagedWidget("brightnessLbl", xmLabelWidgetClass, box4,
-	   NULL);
+	   (XtPointer)NULL);
   brtext = XtVaCreateManagedWidget("brightnessText", xmTextWidgetClass, box4,
-           NULL);
+           (XtPointer)NULL);
 
-  revBtn = XtVaCreateManagedWidget("reverse", xmToggleButtonWidgetClass, box4, NULL);
+  revBtn = XtVaCreateManagedWidget("reverse", xmToggleButtonWidgetClass, box4, (XtPointer)NULL);
 
-  borderBtn = XtVaCreateManagedWidget("borders", xmToggleButtonWidgetClass, box4, NULL);
+  borderBtn = XtVaCreateManagedWidget("borders", xmToggleButtonWidgetClass, box4, (XtPointer)NULL);
 
 
-  box5 = XtVaCreateManagedWidget("box5", xmFormWidgetClass, dialog, NULL);
+  box5 = XtVaCreateManagedWidget("box5", xmFormWidgetClass, dialog, (XtPointer)NULL);
 
            XtVaCreateManagedWidget("outputLbl", xmLabelWidgetClass, box5,
-	   NULL);
+	   (XtPointer)NULL);
 
   outBtns = XmCreateRadioBox(box5, "outputtype", NULL, 0);
 	   XtManageChild(outBtns);
 
 
   button = XtVaCreateManagedWidget("file", xmToggleButtonWidgetClass, outBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, outValueChanged,
-	   TOFILE);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)outValueChanged,
+	   (XtPointer)TOFILE);
 
   filenm = XtVaCreateManagedWidget("fileText", xmTextWidgetClass, box5,
-           NULL);
+           (XtPointer)NULL);
 
   button = XtVaCreateManagedWidget("printer", xmToggleButtonWidgetClass, outBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, outValueChanged,
-	   TOPRN);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)outValueChanged,
+	   (XtPointer)TOPRN);
 
   prncmd = XtVaCreateManagedWidget("printerText", xmTextWidgetClass, box5,
-           NULL);
+           (XtPointer)NULL);
 
 
   cnvBtns = XmCreateRadioBox(dialog, "converttype", NULL, 0);
 
            XtVaSetValues(cnvBtns,
 	   XmNorientation, XmHORIZONTAL,
-	   NULL);
+	   (XtPointer)NULL);
 	   XtManageChild(cnvBtns);
 	   
   button = XtVaCreateManagedWidget("nodither", xmToggleButtonWidgetClass, cnvBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, cnvValueChanged,
-	   NO_DITHER);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)cnvValueChanged,
+	   (XtPointer)NO_DITHER);
 
   button = XtVaCreateManagedWidget("dither", xmToggleButtonWidgetClass, cnvBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, cnvValueChanged,
-	   DITHER);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)cnvValueChanged,
+	   (XtPointer)DITHER);
 
   button = XtVaCreateManagedWidget("mdither", xmToggleButtonWidgetClass, cnvBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, cnvValueChanged,
-	   MDITHER);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)cnvValueChanged,
+	   (XtPointer)MDITHER);
 
   button = XtVaCreateManagedWidget("halftone", xmToggleButtonWidgetClass, cnvBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, cnvValueChanged,
-	   HALFTONE);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)cnvValueChanged,
+	   (XtPointer)HALFTONE);
 
   button = XtVaCreateManagedWidget("mapbw", xmToggleButtonWidgetClass, cnvBtns,
-	   NULL);
-           XtAddCallback(button, XmNvalueChangedCallback, cnvValueChanged,
-	   MAPBW);
+	   (XtPointer)NULL);
+           XtAddCallback(button, XmNvalueChangedCallback, (XtCallbackProc)cnvValueChanged,
+	   (XtPointer)MAPBW);
 
-  box6 = XtVaCreateManagedWidget("box6", xmFormWidgetClass, dialog, NULL);
+  box6 = XtVaCreateManagedWidget("box6", xmFormWidgetClass, dialog, (XtPointer)NULL);
 
-  XtVaCreateManagedWidget("splat", xmLabelWidgetClass, box6, NULL);
+  /* XtVaCreateManagedWidget("splat", xmLabelWidgetClass, box6, (XtPointer)NULL); */
   
-  button = XtVaCreateManagedWidget("OK", xmPushButtonWidgetClass, box6, NULL);
-  XtAddCallback(button, XmNactivateCallback, doGrab, NULL);
+  button = XtVaCreateManagedWidget("OK", xmPushButtonWidgetClass, box6, (XtPointer)NULL);
+  XtAddCallback(button, XmNactivateCallback, (XtCallbackProc)doGrab, (XtPointer)NULL);
 
-  button = XtVaCreateManagedWidget("Dismiss", xmPushButtonWidgetClass, box6, NULL);
-  XtAddCallback(button, XmNactivateCallback, doDismiss, NULL);
+  button = XtVaCreateManagedWidget("Dismiss", xmPushButtonWidgetClass, box6, (XtPointer)NULL);
+  XtAddCallback(button, XmNactivateCallback, (XtCallbackProc)doDismiss, (XtPointer)NULL);
 
+  XtVaCreateManagedWidget("title",  xmLabelWidgetClass, box6,
+           (XtPointer)NULL);
 }
 
 
-
 int main(argc, argv, envp)
   int argc;
   char *argv[];
   char *envp;
 {
   puts("X-Windows Screen Grabber");
+  printf("Version %s\n", version);
   puts(Copyright);
   puts("");
 
 
+
+
+#if defined(OLD_MOTIF)
+  XtAddConverter(XtRString, XtRWindow, XmuCvtStringToWidget,
+		 parentCvtArgs, XtNumber(parentCvtArgs));
+#endif
+
   hShell = XtAppInitialize(&appContext, "XGrab", NULL, 0, &argc, argv,
      fallback_resources, NULL, 0);
 
